@@ -1,4 +1,4 @@
-(* $Id: frontend.ml 76 2005-11-01 00:32:12Z gerd $
+(* $Id: frontend.ml 116 2007-11-11 22:40:21Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -174,13 +174,17 @@ let is_dll p =
 
 
 let identify_dir d =
-  let s = Unix.stat d in
-  (s.Unix.st_dev, s.Unix.st_ino)
+  match Sys.os_type with
+    | "Win32" ->
+	failwith "identify_dir"   (* not available *)
+    | _ ->
+	let s = Unix.stat d in
+	(s.Unix.st_dev, s.Unix.st_ino)
 ;;
 
 
 let conflict_report incpath pkglist =
-  (* First check whether there are several definitions for packages
+  (* Check whether there are several definitions for packages
    * in the current path. We remove duplicate directories first.
    * Note that all other checks are not sensitive to duplicate directories.
    *)
@@ -268,9 +272,19 @@ let run_command ?filter verbose cmd args =
   let old_sigint =
     Sys.signal Sys.sigint Sys.Signal_ignore in
 
+  let need_exe =
+    List.mem Findlib_config.system [ "win32"; "mingw" ] in
+
+  let fixed_cmd =
+    if need_exe then (
+      if Filename.check_suffix cmd ".exe" then cmd else cmd ^ ".exe" 
+    )
+    else
+      cmd in
+
   let pid =
     Unix.create_process
-      cmd
+      fixed_cmd
       (Array.of_list (cmd :: args))
       Unix.stdin
       cmd_output
@@ -376,7 +390,11 @@ let query_package () =
   let i_format =
     "-I %d" in
   let l_format =
-    "-ccopt -L%d" in
+    if Findlib_config.system = "win32" then
+      (* Microsoft toolchain *)
+      "-ccopt \"/link /libpath:%d\""
+    else
+      "-ccopt -L%d" in
   let a_format =
     "%a" in
   let o_format =
@@ -468,9 +486,42 @@ let process_pp_spec syntax_preds packages pp_opts =
   (* Returns: pp_command *)
   (* may raise No_such_package *)
 
+  (* [packages]: all packages given on the command line. May include
+   * packages for compilation and for preprocessing.
+   *
+   * The difficulty is now that the preprocessor packages may have
+   * requirements that are non-preprocessor packages. To get exactly
+   * the preprocessor packages and its requirements, we do:
+   *
+   * 1. Determine the subset of [packages] that are preprocessor
+   *    packages by checking whether they have an "archive" for
+   *    [syntax_preds], i.e. the preprocessor packages mentioned
+   *    on the command line = [cl_pp_packages].
+   *
+   * 2. Add their requirements = [pp_packages]
+   *
+   * Because the packages are now mixed, we must evaluate for 
+   * [syntax_preds] + "byte".
+   *)
+
+  let cl_pp_packages =
+    if syntax_preds = [] then
+      (* No syntax predicates, no preprocessor! *)
+      []
+    else
+      List.filter
+	(fun pkg ->
+	   let al = try package_property syntax_preds pkg "archive"
+	            with Not_found -> "" in
+	   let w = Fl_split.in_words al in
+	   w <> []
+	)
+	packages in
+
   let pp_packages =
-    package_deep_ancestors syntax_preds packages in
-  (* the packages used for the preprocessor *)
+    package_deep_ancestors syntax_preds cl_pp_packages in
+
+  (* One packages must now have the variable "preprocessor", usually camlp4 *)
 
   let preprocessor_cmds =
     List.flatten
@@ -521,16 +572,10 @@ let process_pp_spec syntax_preds packages pp_opts =
       List.flatten
 	(List.map
 	   (fun pkg ->
-	      let al = try package_property syntax_preds pkg "archive"
-	               with Not_found -> "" in
-	      let w = Fl_split.in_words al in
-	      if w = [] then
-		let al' =
-		  try package_property ("byte" :: syntax_preds) pkg "archive"
-	          with Not_found -> "" in
-		Fl_split.in_words al'
-	      else
-		w
+	      let al = 
+		try package_property ("byte" :: syntax_preds) pkg "archive"
+	        with Not_found -> "" in
+	      Fl_split.in_words al
 	   )
 	   pp_packages) in
 
@@ -677,7 +722,7 @@ let ocamlc which () =
 	  "-I", (Arg.String
 		   (fun s ->
 		      let s = resolve_path s in
-		      incpath := s :: !incpath;  (* reverted below *)
+                      if Sys.file_exists s then incpath := s :: !incpath;  (* reverted below *)
 		      add_spec_fn "-I" s ));
 
 	  "-impl", 
@@ -883,7 +928,11 @@ let ocamlc which () =
 	    if List.mem npkgdir exclude_list then
 	      []
 	    else
-	      [ "-ccopt"; "-L" ^ pkgdir; ])
+	      if Findlib_config.system = "win32" then
+		(* Microsoft toolchain *)
+		[ "-ccopt"; "/link /libpath:" ^ pkgdir ]
+	      else
+		[ "-ccopt"; "-L" ^ pkgdir; ])
 	 eff_link_dl) in
 
   let archives =
